window.Gleam.initSearch([{"doc":"json_blueprint","title":"json_blueprint","content":"# json_blueprint\n\njson_blueprint is a Gleam library that simplifies JSON encoding and decoding while automatically generating JSON schemas for your data types.\n\n[![Package Version](https://img.shields.io/hexpm/v/json_blueprint)](https://hex.pm/packages/json_blueprint)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/json_blueprint/)\n\n```sh\ngleam add json_blueprint\n```\n\n## Usage\n\njson_blueprint provides utilities for encoding and decoding JSON data, with special support for union types. The generated JSON schemas can be used to validate incoming JSON data with the decoder. The JSON schema follows the [JSON Schema Draft 7](https://json-schema.org/) specification and can tested and validate on [JSON Schema Lint](https://jsonschemalint.com/#!/version/draft-07/markup/json).\n\n&gt; ❗️ _**IMPORTANT: Recursive data types**_\n&gt;\n&gt; Make to use the `self_decoder` when defining the decoder for recursive data types.\n\n&gt; ⚠️ _**WARNING: Do NOT use on cyclical data type definitions**_\n&gt;\n&gt; While the library supports recursive data types (types with self reference), it does not support cyclical data types (cyclical dependency between multiple data types). Cyclical data types will result in infinite loop during decoding or schema generation.\n\n## Examples\n\n&lt;details&gt;\n  &lt;summary&gt;Encoding Union Types&lt;/summary&gt;\n  \nHere&#39;s an example of encoding a union type to JSON:\n\n```gleam\nimport gleam/io\nimport gleam/json\nimport gleeunit\nimport gleeunit/should\nimport json/blueprint\n\npub fn main() {\n  gleeunit.main()\n}\n\ntype Shape {\n  Circle(Float)\n  Rectangle(Float, Float)\n  Void\n}\n\nfn encode_shape(shape: Shape) -&gt; json.Json {\n  blueprint.union_type_encoder(shape, fn(shape_case) {\n    case shape_case {\n      Circle(radius) -&gt; #(\n        &quot;circle&quot;,\n        json.object([#(&quot;radius&quot;, json.float(radius))]),\n      )\n      Rectangle(width, height) -&gt; #(\n        &quot;rectangle&quot;,\n        json.object([\n          #(&quot;width&quot;, json.float(width)),\n          #(&quot;height&quot;, json.float(height)),\n        ]),\n      )\n      Void -&gt; #(&quot;void&quot;, json.object([]))\n    }\n  })\n}\n\nfn shape_decoder() -&gt; blueprint.Decoder(Shape) {\n  blueprint.union_type_decoder([\n    #(\n      &quot;circle&quot;,\n      blueprint.decode1(Circle, blueprint.field(&quot;radius&quot;, blueprint.float())),\n    ),\n    #(\n      &quot;rectangle&quot;,\n      blueprint.decode2(\n        Rectangle,\n        blueprint.field(&quot;width&quot;, blueprint.float()),\n        blueprint.field(&quot;height&quot;, blueprint.float()),\n      ),\n    ),\n    #(&quot;void&quot;, blueprint.decode0(Void)),\n  ])\n}\n\npub fn union_type_test() {\n  let circle = Circle(5.0)\n  let rectangle = Rectangle(10.0, 20.0)\n\n  let decoder = shape_decoder()\n\n  //test decoding\n  encode_shape(circle)\n  |&gt; json.to_string\n  |&gt; blueprint.decode(using: decoder)\n  |&gt; should.equal(Ok(circle))\n\n  encode_shape(rectangle)\n  |&gt; json.to_string\n  |&gt; blueprint.decode(using: decoder)\n  |&gt; should.equal(Ok(rectangle))\n\n  encode_shape(Void)\n  |&gt; json.to_string\n  |&gt; blueprint.decode(using: decoder)\n  |&gt; should.equal(Ok(Void))\n\n  blueprint.generate_json_schema(shape_decoder())\n  |&gt; json.to_string\n  |&gt; io.println\n}\n```\n\n#### Generated JSON Schema\n\n```json\n{\n  &quot;$schema&quot;: &quot;http://json-schema.org/draft-07/schema#&quot;,\n  &quot;anyOf&quot;: [\n    {\n      &quot;required&quot;: [&quot;type&quot;, &quot;data&quot;],\n      &quot;additionalProperties&quot;: false,\n      &quot;type&quot;: &quot;object&quot;,\n      &quot;properties&quot;: {\n        &quot;type&quot;: {\n          &quot;type&quot;: &quot;string&quot;,\n          &quot;enum&quot;: [&quot;circle&quot;]\n        },\n        &quot;data&quot;: {\n          &quot;required&quot;: [&quot;radius&quot;],\n          &quot;additionalProperties&quot;: false,\n          &quot;type&quot;: &quot;object&quot;,\n          &quot;properties&quot;: {\n            &quot;radius&quot;: {\n              &quot;type&quot;: &quot;number&quot;\n            }\n          }\n        }\n      }\n    },\n    {\n      &quot;required&quot;: [&quot;type&quot;, &quot;data&quot;],\n      &quot;additionalProperties&quot;: false,\n      &quot;type&quot;: &quot;object&quot;,\n      &quot;properties&quot;: {\n        &quot;type&quot;: {\n          &quot;type&quot;: &quot;string&quot;,\n          &quot;enum&quot;: [&quot;rectangle&quot;]\n        },\n        &quot;data&quot;: {\n          &quot;required&quot;: [&quot;width&quot;, &quot;height&quot;],\n          &quot;additionalProperties&quot;: false,\n          &quot;type&quot;: &quot;object&quot;,\n          &quot;properties&quot;: {\n            &quot;width&quot;: {\n              &quot;type&quot;: &quot;number&quot;\n            },\n            &quot;height&quot;: {\n              &quot;type&quot;: &quot;number&quot;\n            }\n          }\n        }\n      }\n    },\n    {\n      &quot;required&quot;: [&quot;type&quot;, &quot;data&quot;],\n      &quot;additionalProperties&quot;: false,\n      &quot;type&quot;: &quot;object&quot;,\n      &quot;properties&quot;: {\n        &quot;type&quot;: {\n          &quot;type&quot;: &quot;string&quot;,\n          &quot;enum&quot;: [&quot;void&quot;]\n        },\n        &quot;data&quot;: {\n          &quot;additionalProperties&quot;: false,\n          &quot;type&quot;: &quot;object&quot;,\n          &quot;properties&quot;: {}\n        }\n      }\n    }\n  ]\n}\n```\n\nThis will encode your union types into a standardized JSON format with `type` and `data` fields, making it easy to decode on the receiving end.\n\n&lt;/details&gt;\n\n&lt;details&gt;\n  &lt;summary&gt;Type aliases and optional fields&lt;/summary&gt;\n  \nAnd here&#39;s an example using type aliases, optional fields, and single constructor types:\n\n```gleam\nimport gleam/io\nimport gleam/json\nimport gleam/option.{type Option, None, Some}\nimport gleeunit\nimport gleeunit/should\nimport json/blueprint\n\npub fn main() {\n  gleeunit.main()\n}\n\ntype Color {\n  Red\n  Green\n  Blue\n}\n\ntype Coordinate =\n  #(Float, Float)\n\ntype Drawing {\n  Box(Float, Float, Coordinate, Option(Color))\n}\n\nfn color_decoder() {\n  blueprint.enum_type_decoder([\n    #(&quot;red&quot;, Red),\n    #(&quot;green&quot;, Green),\n    #(&quot;blue&quot;, Blue),\n  ])\n}\n\nfn color_encoder(input) {\n  blueprint.enum_type_encoder(input, fn(color) {\n    case color {\n      Red -&gt; &quot;red&quot;\n      Green -&gt; &quot;green&quot;\n      Blue -&gt; &quot;blue&quot;\n    }\n  })\n}\n\nfn encode_coordinate(coord: Coordinate) -&gt; json.Json {\n  blueprint.encode_tuple2(coord, json.float, json.float)\n}\n\nfn coordinate_decoder() {\n  blueprint.tuple2(blueprint.float(), blueprint.float())\n}\n\nfn encode_drawing(drawing: Drawing) -&gt; json.Json {\n  blueprint.union_type_encoder(drawing, fn(shape) {\n    case shape {\n      Box(width, height, position, color) -&gt; #(\n        &quot;box&quot;,\n        json.object([\n          #(&quot;width&quot;, json.float(width)),\n          #(&quot;height&quot;, json.float(height)),\n          #(&quot;position&quot;, encode_coordinate(position)),\n          #(&quot;color&quot;, json.nullable(color, color_encoder)),\n        ]),\n      )\n    }\n  })\n}\n\nfn drawing_decoder() -&gt; blueprint.Decoder(Drawing) {\n  blueprint.union_type_decoder([\n    #(\n      &quot;box&quot;,\n      blueprint.decode4(\n        Box,\n        blueprint.field(&quot;width&quot;, blueprint.float()),\n        blueprint.field(&quot;height&quot;, blueprint.float()),\n        blueprint.field(&quot;position&quot;, coordinate_decoder()),\n        blueprint.optional_field(&quot;color&quot;, color_decoder()),\n      ),\n    ),\n  ])\n}\n\npub fn drawing_test() {\n  // Test cases\n  let box = Box(15.0, 25.0, #(30.0, 40.0), None)\n\n  // Test encoding\n  let encoded_box = encode_drawing(box)\n\n  // Test decoding\n  encoded_box\n  |&gt; json.to_string\n  |&gt; blueprint.decode(using: drawing_decoder())\n  |&gt; should.equal(Ok(box))\n\n  blueprint.generate_json_schema(drawing_decoder())\n  |&gt; json.to_string\n  |&gt; io.println\n}\n\n```\n\n#### Generated JSON Schema\n\n```json\n{\n  &quot;$schema&quot;: &quot;http://json-schema.org/draft-07/schema#&quot;,\n  &quot;required&quot;: [&quot;type&quot;, &quot;data&quot;],\n  &quot;additionalProperties&quot;: false,\n  &quot;type&quot;: &quot;object&quot;,\n  &quot;properties&quot;: {\n    &quot;type&quot;: {\n      &quot;type&quot;: &quot;string&quot;,\n      &quot;enum&quot;: [&quot;box&quot;]\n    },\n    &quot;data&quot;: {\n      &quot;required&quot;: [&quot;width&quot;, &quot;height&quot;, &quot;position&quot;],\n      &quot;additionalProperties&quot;: false,\n      &quot;type&quot;: &quot;object&quot;,\n      &quot;properties&quot;: {\n        &quot;width&quot;: {\n          &quot;type&quot;: &quot;number&quot;\n        },\n        &quot;height&quot;: {\n          &quot;type&quot;: &quot;number&quot;\n        },\n        &quot;position&quot;: {\n          &quot;maxItems&quot;: 2,\n          &quot;minItems&quot;: 2,\n          &quot;prefixItems&quot;: [\n            {\n              &quot;type&quot;: &quot;number&quot;\n            },\n            {\n              &quot;type&quot;: &quot;number&quot;\n            }\n          ],\n          &quot;type&quot;: &quot;array&quot;\n        },\n        &quot;color&quot;: {\n          &quot;required&quot;: [&quot;enum&quot;],\n          &quot;additionalProperties&quot;: false,\n          &quot;type&quot;: &quot;object&quot;,\n          &quot;properties&quot;: {\n            &quot;enum&quot;: {\n              &quot;type&quot;: &quot;string&quot;,\n              &quot;enum&quot;: [&quot;red&quot;, &quot;green&quot;, &quot;blue&quot;]\n            }\n          }\n        }\n      }\n    }\n  }\n}\n```\n\n&lt;/details&gt;\n\n&lt;details&gt;\n  &lt;summary&gt;Recursive data types&lt;/summary&gt;\n  \nAnd here&#39;s an example using type aliases, optional fields, and single constructor types:\n\n```gleam\nimport gleam/io\nimport gleam/json\nimport gleam/option.{type Option, None, Some}\nimport gleeunit\nimport gleeunit/should\nimport json/blueprint\n\npub fn main() {\n  gleeunit.main()\n}\n\ntype Tree {\n  Node(value: Int, left: Option(Tree), right: Option(Tree))\n}\n\ntype ListOfTrees(t) {\n  ListOfTrees(head: t, tail: ListOfTrees(t))\n  NoTrees\n}\n\nfn encode_tree(tree: Tree) -&gt; json.Json {\n  blueprint.union_type_encoder(tree, fn(node) {\n    case node {\n      Node(value, left, right) -&gt; #(\n        &quot;node&quot;,\n        [\n          #(&quot;value&quot;, json.int(value)),\n          #(&quot;right&quot;, json.nullable(right, encode_tree)),\n        ]\n          |&gt; blueprint.encode_optional_field(&quot;left&quot;, left, encode_tree)\n          |&gt; json.object(),\n      )\n    }\n  })\n}\n\nfn encode_list_of_trees(tree: ListOfTrees(Tree)) -&gt; json.Json {\n  blueprint.union_type_encoder(tree, fn(list) {\n    case list {\n      ListOfTrees(head, tail) -&gt; #(\n        &quot;list&quot;,\n        json.object([\n          #(&quot;head&quot;, encode_tree(head)),\n          #(&quot;tail&quot;, encode_list_of_trees(tail)),\n        ]),\n      )\n      NoTrees -&gt; #(&quot;no_trees&quot;, json.object([]))\n    }\n  })\n}\n\n// Without reuse_decoder, recursive types would cause infinite schema expansion\nfn tree_decoder() {\n  blueprint.union_type_decoder([\n    #(\n      &quot;node&quot;,\n      blueprint.decode3(\n        Node,\n        blueprint.field(&quot;value&quot;, blueprint.int()),\n        // testing both an optional field a field with a possible null\n        blueprint.optional_field(&quot;left&quot;, blueprint.self_decoder(tree_decoder)),\n        blueprint.field(\n          &quot;right&quot;,\n          blueprint.optional(blueprint.self_decoder(tree_decoder)),\n        ),\n      ),\n    ),\n  ])\n  // !!!IMPORTANT!!! Add the reuse_decoder when there are nested recursive types so\n  // the schema references (`#`) get rewritten correctly and self-references from the\n  // different types don&#39;t get mixed up. As a recommendation, always add it when\n  // decoding recursive types.\n  |&gt; blueprint.reuse_decoder\n}\n\nfn decode_list_of_trees() {\n  blueprint.union_type_decoder([\n    #(\n      &quot;list&quot;,\n      blueprint.decode2(\n        ListOfTrees,\n        blueprint.field(&quot;head&quot;, tree_decoder()),\n        blueprint.field(&quot;tail&quot;, blueprint.self_decoder(decode_list_of_trees)),\n      ),\n    ),\n    #(&quot;no_trees&quot;, blueprint.decode0(NoTrees)),\n  ])\n}\n\npub fn tree_decoder_test() {\n  // Create a sample tree structure:\n  //       5\n  //      / \\\n  //     3   7\n  //    /     \\\n  //   1       9\n  let tree =\n    Node(\n      value: 5,\n      left: Some(Node(value: 3, left: Some(Node(1, None, None)), right: None)),\n      right: Some(Node(value: 7, left: None, right: Some(Node(9, None, None)))),\n    )\n\n  // Create a list of trees\n  let tree_list =\n    ListOfTrees(\n      Node(value: 1, left: None, right: None),\n      ListOfTrees(\n        Node(\n          value: 10,\n          left: Some(Node(value: 1, left: None, right: None)),\n          right: None,\n        ),\n        NoTrees,\n      ),\n    )\n\n  // Test encoding\n  let json_str = tree |&gt; encode_tree |&gt; json.to_string()\n  let list_json_str = tree_list |&gt; encode_list_of_trees |&gt; json.to_string()\n\n  // Test decoding\n  let decoded = blueprint.decode(using: tree_decoder(), from: json_str)\n\n  decoded\n  |&gt; should.equal(Ok(tree))\n\n  let decoded_list =\n    blueprint.decode(using: decode_list_of_trees(), from: list_json_str)\n\n  decoded_list\n  |&gt; should.equal(Ok(tree_list))\n\n  // Test schema generation\n  blueprint.generate_json_schema(decode_list_of_trees())\n  |&gt; json.to_string\n  |&gt; io.println\n}\n```\n\n#### Generated JSON Schema\n\n```json\n{\n  &quot;$defs&quot;: {\n    &quot;ref_CEF475B4CA96DC7B2C0C206AC7598AFFC4B66FD2&quot;: {\n      &quot;required&quot;: [&quot;type&quot;, &quot;data&quot;],\n      &quot;additionalProperties&quot;: false,\n      &quot;type&quot;: &quot;object&quot;,\n      &quot;properties&quot;: {\n        &quot;type&quot;: {\n          &quot;type&quot;: &quot;string&quot;,\n          &quot;enum&quot;: [&quot;node&quot;]\n        },\n        &quot;data&quot;: {\n          &quot;required&quot;: [&quot;value&quot;, &quot;right&quot;],\n          &quot;additionalProperties&quot;: false,\n          &quot;type&quot;: &quot;object&quot;,\n          &quot;properties&quot;: {\n            &quot;value&quot;: {\n              &quot;type&quot;: &quot;integer&quot;\n            },\n            &quot;left&quot;: {\n              &quot;$ref&quot;: &quot;#/$defs/ref_CEF475B4CA96DC7B2C0C206AC7598AFFC4B66FD2&quot;\n            },\n            &quot;right&quot;: {\n              &quot;anyOf&quot;: [\n                {\n                  &quot;$ref&quot;: &quot;#/$defs/ref_CEF475B4CA96DC7B2C0C206AC7598AFFC4B66FD2&quot;\n                },\n                {\n                  &quot;type&quot;: &quot;null&quot;\n                }\n              ]\n            }\n          }\n        }\n      }\n    }\n  },\n  &quot;$schema&quot;: &quot;http://json-schema.org/draft-07/schema#&quot;,\n  &quot;anyOf&quot;: [\n    {\n      &quot;required&quot;: [&quot;type&quot;, &quot;data&quot;],\n      &quot;additionalProperties&quot;: false,\n      &quot;type&quot;: &quot;object&quot;,\n      &quot;properties&quot;: {\n        &quot;type&quot;: {\n          &quot;type&quot;: &quot;string&quot;,\n          &quot;enum&quot;: [&quot;list&quot;]\n        },\n        &quot;data&quot;: {\n          &quot;required&quot;: [&quot;head&quot;, &quot;tail&quot;],\n          &quot;additionalProperties&quot;: false,\n          &quot;type&quot;: &quot;object&quot;,\n          &quot;properties&quot;: {\n            &quot;head&quot;: {\n              &quot;$ref&quot;: &quot;#/$defs/ref_CEF475B4CA96DC7B2C0C206AC7598AFFC4B66FD2&quot;\n            },\n            &quot;tail&quot;: {\n              &quot;$ref&quot;: &quot;#&quot;\n            }\n          }\n        }\n      }\n    },\n    {\n      &quot;required&quot;: [&quot;type&quot;, &quot;data&quot;],\n      &quot;additionalProperties&quot;: false,\n      &quot;type&quot;: &quot;object&quot;,\n      &quot;properties&quot;: {\n        &quot;type&quot;: {\n          &quot;type&quot;: &quot;string&quot;,\n          &quot;enum&quot;: [&quot;no_trees&quot;]\n        },\n        &quot;data&quot;: {\n          &quot;additionalProperties&quot;: false,\n          &quot;type&quot;: &quot;object&quot;,\n          &quot;properties&quot;: {}\n        }\n      }\n    }\n  ]\n}\n```\n\n&lt;/details&gt;\n\n## Features\n\n- 🎯 Type-safe JSON encoding and decoding\n- 🔄 Support for union types with standardized encoding\n- 📋 Automatic JSON schema generation\n- ✨ Clean and intuitive API\n\nFurther documentation can be found at &lt;https://hexdocs.pm/json_blueprint&gt;.\n\n## Development\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the tests\n```\n","url":"index.html"},{"doc":"json/blueprint/schema","title":"Schema","content":"pub type Schema {\n  Schema(\n    schema: SchemaDefinition,\n    vocabulary: Option(List(String)),\n    id: Option(String),\n    comment: Option(String),\n    defs: Option(List(#(String, SchemaDefinition))),\n  )\n}\n Represents a complete JSON Schema document\nSchema(\n  schema: SchemaDefinition,\n  vocabulary: Option(List(String)),\n  id: Option(String),\n  comment: Option(String),\n  defs: Option(List(#(String, SchemaDefinition))),\n)\n\n\nSynonyms:\njson/blueprint/schema.Schema\njson/blueprint/schema Schema","url":"json/blueprint/schema.html#Schema"},{"doc":"json/blueprint/schema","title":"SchemaDefinition","content":"pub type SchemaDefinition {\n  Type(type_: SchemaType)\n  Enum(values: List(json.Json), type_: Option(SchemaType))\n  Const(value: json.Json)\n  Nullable(schema: SchemaDefinition)\n  Optional(schema: SchemaDefinition)\n  Number(\n    minimum: Option(Float),\n    maximum: Option(Float),\n    exclusive_minimum: Option(Float),\n    exclusive_maximum: Option(Float),\n    multiple_of: Option(Float),\n  )\n  String(\n    min_length: Option(Int),\n    max_length: Option(Int),\n    pattern: Option(String),\n    format: Option(StringFormat),\n  )\n  Array(items: Option(SchemaDefinition))\n  DetailedArray(\n    items: Option(SchemaDefinition),\n    prefix_items: Option(List(SchemaDefinition)),\n    min_items: Option(Int),\n    max_items: Option(Int),\n    unique_items: Option(Bool),\n    contains: Option(SchemaDefinition),\n    min_contains: Option(Int),\n    max_contains: Option(Int),\n  )\n  Object(\n    properties: List(#(String, SchemaDefinition)),\n    additional_properties: Option(Bool),\n    required: Option(List(String)),\n  )\n  DetailedObject(\n    properties: Option(List(#(String, SchemaDefinition))),\n    pattern_properties: Option(List(#(String, SchemaDefinition))),\n    additional_properties: Option(SchemaDefinition),\n    required: Option(List(String)),\n    property_names: Option(SchemaDefinition),\n    min_properties: Option(Int),\n    max_properties: Option(Int),\n  )\n  AllOf(schemas: List(SchemaDefinition))\n  AnyOf(schemas: List(SchemaDefinition))\n  OneOf(schemas: List(SchemaDefinition))\n  Not(schema: SchemaDefinition)\n  Ref(ref: String)\n  TrueValue\n  FalseValue\n}\n Core schema definition that can be used recursively\nType(type_: SchemaType)\n\n\nEnum(values: List(json.Json), type_: Option(SchemaType))\n\n\nConst(value: json.Json)\n\n\nNullable(schema: SchemaDefinition)\n\n\nOptional(schema: SchemaDefinition)\n\n\nNumber(\n  minimum: Option(Float),\n  maximum: Option(Float),\n  exclusive_minimum: Option(Float),\n  exclusive_maximum: Option(Float),\n  multiple_of: Option(Float),\n)\n\n\nString(\n  min_length: Option(Int),\n  max_length: Option(Int),\n  pattern: Option(String),\n  format: Option(StringFormat),\n)\n\n\nArray(items: Option(SchemaDefinition))\n\n\nDetailedArray(\n  items: Option(SchemaDefinition),\n  prefix_items: Option(List(SchemaDefinition)),\n  min_items: Option(Int),\n  max_items: Option(Int),\n  unique_items: Option(Bool),\n  contains: Option(SchemaDefinition),\n  min_contains: Option(Int),\n  max_contains: Option(Int),\n)\n\n\nObject(\n  properties: List(#(String, SchemaDefinition)),\n  additional_properties: Option(Bool),\n  required: Option(List(String)),\n)\n\n\nDetailedObject(\n  properties: Option(List(#(String, SchemaDefinition))),\n  pattern_properties: Option(List(#(String, SchemaDefinition))),\n  additional_properties: Option(SchemaDefinition),\n  required: Option(List(String)),\n  property_names: Option(SchemaDefinition),\n  min_properties: Option(Int),\n  max_properties: Option(Int),\n)\n\n\nAllOf(schemas: List(SchemaDefinition))\n\n\nAnyOf(schemas: List(SchemaDefinition))\n\n\nOneOf(schemas: List(SchemaDefinition))\n\n\nNot(schema: SchemaDefinition)\n\n\nRef(ref: String)\n\n\nTrueValue\n\n\nFalseValue\n\n\nSynonyms:\njson/blueprint/schema.SchemaDefinition\njson/blueprint/schema SchemaDefinition","url":"json/blueprint/schema.html#SchemaDefinition"},{"doc":"json/blueprint/schema","title":"SchemaType","content":"pub type SchemaType {\n  Null\n  BooleanType\n  ObjectType\n  ArrayType\n  NumberType\n  StringType\n  IntegerType\n  Multiple(List(SchemaType))\n}\n Represents the allowed basic JSON Schema types\nNull\n\n\nBooleanType\n\n\nObjectType\n\n\nArrayType\n\n\nNumberType\n\n\nStringType\n\n\nIntegerType\n\n\nMultiple(List(SchemaType))\n\n\nSynonyms:\njson/blueprint/schema.SchemaType\njson/blueprint/schema SchemaType","url":"json/blueprint/schema.html#SchemaType"},{"doc":"json/blueprint/schema","title":"StringFormat","content":"pub type StringFormat {\n  DateTime\n  Date\n  Time\n  Duration\n  Email\n  IdnEmail\n  Hostname\n  IdnHostname\n  Ipv4\n  Ipv6\n  Uri\n  UriReference\n  Iri\n  IriReference\n  UriTemplate\n  JsonPointer\n  RelativeJsonPointer\n  Regex\n}\n Common string formats defined in the spec\nDateTime\n\n\nDate\n\n\nTime\n\n\nDuration\n\n\nEmail\n\n\nIdnEmail\n\n\nHostname\n\n\nIdnHostname\n\n\nIpv4\n\n\nIpv6\n\n\nUri\n\n\nUriReference\n\n\nIri\n\n\nIriReference\n\n\nUriTemplate\n\n\nJsonPointer\n\n\nRelativeJsonPointer\n\n\nRegex\n\n\nSynonyms:\njson/blueprint/schema.StringFormat\njson/blueprint/schema StringFormat","url":"json/blueprint/schema.html#StringFormat"},{"doc":"json/blueprint/schema","title":"json_schema_version","content":"pub const json_schema_version: String\n\nSynonyms:\njson/blueprint/schema.json_schema_version\njson/blueprint/schema json_schema_version","url":"json/blueprint/schema.html#json_schema_version"},{"doc":"json/blueprint/schema","title":"hash_schema_definition","content":"pub fn hash_schema_definition(def: SchemaDefinition) -&gt; String\n\nSynonyms:\njson/blueprint/schema.hash_schema_definition\njson/blueprint/schema hash_schema_definition","url":"json/blueprint/schema.html#hash_schema_definition"},{"doc":"json/blueprint/schema","title":"map_ref","content":"pub fn map_ref(\n  def: SchemaDefinition,\n  f: fn(String) -&gt; String,\n) -&gt; SchemaDefinition\n\nSynonyms:\njson/blueprint/schema.map_ref\njson/blueprint/schema map_ref","url":"json/blueprint/schema.html#map_ref"},{"doc":"json/blueprint/schema","title":"new_schema","content":"pub fn new_schema(\n  definition: SchemaDefinition,\n  references: Option(List(#(String, SchemaDefinition))),\n) -&gt; Schema\n Helper function to create a new schema with default values\nSynonyms:\njson/blueprint/schema.new_schema\njson/blueprint/schema new_schema","url":"json/blueprint/schema.html#new_schema"},{"doc":"json/blueprint/schema","title":"number_constraint","content":"pub fn number_constraint(\n  minimum minimum: Option(Float),\n  maximum maximum: Option(Float),\n  exclusive_minimum exclusive_minimum: Option(Float),\n  exclusive_maximum exclusive_maximum: Option(Float),\n  multiple_of multiple_of: Option(Float),\n) -&gt; SchemaDefinition\n Helper to create a number constraint\nSynonyms:\njson/blueprint/schema.number_constraint\njson/blueprint/schema number_constraint","url":"json/blueprint/schema.html#number_constraint"},{"doc":"json/blueprint/schema","title":"string_constraint","content":"pub fn string_constraint(\n  min_length min_length: Option(Int),\n  max_length max_length: Option(Int),\n  pattern pattern: Option(String),\n  format format: Option(StringFormat),\n) -&gt; SchemaDefinition\n Helper to create a string constraint\nSynonyms:\njson/blueprint/schema.string_constraint\njson/blueprint/schema string_constraint","url":"json/blueprint/schema.html#string_constraint"},{"doc":"json/blueprint/schema","title":"to_json","content":"pub fn to_json(schema: Schema) -&gt; Json\n Convert a Schema to JSON value\nSynonyms:\njson/blueprint/schema.to_json\njson/blueprint/schema to_json","url":"json/blueprint/schema.html#to_json"},{"doc":"json/blueprint/schema","title":"to_json_string","content":"pub fn to_json_string(schema: Schema) -&gt; String\n Convert a Schema to JSON string\nSynonyms:\njson/blueprint/schema.to_json_string\njson/blueprint/schema to_json_string","url":"json/blueprint/schema.html#to_json_string"},{"doc":"json/blueprint/schema","title":"type_constraint","content":"pub fn type_constraint(type_: SchemaType) -&gt; SchemaDefinition\n Helper to create a basic type constraint\nSynonyms:\njson/blueprint/schema.type_constraint\njson/blueprint/schema type_constraint","url":"json/blueprint/schema.html#type_constraint"},{"doc":"json/blueprint/schema","title":"json/blueprint/schema","content":"","url":"json/blueprint/schema.html"},{"doc":"json/blueprint","title":"Decoder","content":"pub type Decoder(t) {\n  Decoder(\n    dyn_decoder: dynamic.Decoder(t),\n    schema: SchemaDefinition,\n    defs: List(#(String, SchemaDefinition)),\n  )\n}\n\nDecoder(\n  dyn_decoder: dynamic.Decoder(t),\n  schema: SchemaDefinition,\n  defs: List(#(String, SchemaDefinition)),\n)\n\n\nSynonyms:\njson/blueprint.Decoder\njson/blueprint Decoder","url":"json/blueprint.html#Decoder"},{"doc":"json/blueprint","title":"FieldDecoder","content":"pub type FieldDecoder(t) {\n  FieldDecoder(\n    dyn_decoder: dynamic.Decoder(t),\n    field_schema: #(String, SchemaDefinition),\n    defs: List(#(String, SchemaDefinition)),\n  )\n}\n\nFieldDecoder(\n  dyn_decoder: dynamic.Decoder(t),\n  field_schema: #(String, SchemaDefinition),\n  defs: List(#(String, SchemaDefinition)),\n)\n\n\nSynonyms:\njson/blueprint.FieldDecoder\njson/blueprint FieldDecoder","url":"json/blueprint.html#FieldDecoder"},{"doc":"json/blueprint","title":"LazyDecoder","content":"pub type LazyDecoder(t) =\n  fn() -&gt; Decoder(t)\n\n\nSynonyms:\njson/blueprint.LazyDecoder\njson/blueprint LazyDecoder","url":"json/blueprint.html#LazyDecoder"},{"doc":"json/blueprint","title":"bool","content":"pub fn bool() -&gt; Decoder(Bool)\n\nSynonyms:\njson/blueprint.bool\njson/blueprint bool","url":"json/blueprint.html#bool"},{"doc":"json/blueprint","title":"decode","content":"pub fn decode(\n  using decoder: Decoder(a),\n  from json_string: String,\n) -&gt; Result(a, DecodeError)\n\nSynonyms:\njson/blueprint.decode\njson/blueprint decode","url":"json/blueprint.html#decode"},{"doc":"json/blueprint","title":"decode0","content":"pub fn decode0(constructor: a) -&gt; Decoder(a)\n\nSynonyms:\njson/blueprint.decode0\njson/blueprint decode0","url":"json/blueprint.html#decode0"},{"doc":"json/blueprint","title":"decode1","content":"pub fn decode1(\n  constructor: fn(a) -&gt; b,\n  t1: FieldDecoder(a),\n) -&gt; Decoder(b)\n\nSynonyms:\njson/blueprint.decode1\njson/blueprint decode1","url":"json/blueprint.html#decode1"},{"doc":"json/blueprint","title":"decode2","content":"pub fn decode2(\n  constructor: fn(a, b) -&gt; c,\n  t1: FieldDecoder(a),\n  t2: FieldDecoder(b),\n) -&gt; Decoder(c)\n\nSynonyms:\njson/blueprint.decode2\njson/blueprint decode2","url":"json/blueprint.html#decode2"},{"doc":"json/blueprint","title":"decode3","content":"pub fn decode3(\n  constructor: fn(a, b, c) -&gt; d,\n  t1: FieldDecoder(a),\n  t2: FieldDecoder(b),\n  t3: FieldDecoder(c),\n) -&gt; Decoder(d)\n\nSynonyms:\njson/blueprint.decode3\njson/blueprint decode3","url":"json/blueprint.html#decode3"},{"doc":"json/blueprint","title":"decode4","content":"pub fn decode4(\n  constructor: fn(a, b, c, d) -&gt; e,\n  t1: FieldDecoder(a),\n  t2: FieldDecoder(b),\n  t3: FieldDecoder(c),\n  t4: FieldDecoder(d),\n) -&gt; Decoder(e)\n\nSynonyms:\njson/blueprint.decode4\njson/blueprint decode4","url":"json/blueprint.html#decode4"},{"doc":"json/blueprint","title":"decode5","content":"pub fn decode5(\n  constructor: fn(a, b, c, d, e) -&gt; f,\n  t1: FieldDecoder(a),\n  t2: FieldDecoder(b),\n  t3: FieldDecoder(c),\n  t4: FieldDecoder(d),\n  t5: FieldDecoder(e),\n) -&gt; Decoder(f)\n\nSynonyms:\njson/blueprint.decode5\njson/blueprint decode5","url":"json/blueprint.html#decode5"},{"doc":"json/blueprint","title":"decode6","content":"pub fn decode6(\n  constructor: fn(a, b, c, d, e, f) -&gt; g,\n  t1: FieldDecoder(a),\n  t2: FieldDecoder(b),\n  t3: FieldDecoder(c),\n  t4: FieldDecoder(d),\n  t5: FieldDecoder(e),\n  t6: FieldDecoder(f),\n) -&gt; Decoder(g)\n\nSynonyms:\njson/blueprint.decode6\njson/blueprint decode6","url":"json/blueprint.html#decode6"},{"doc":"json/blueprint","title":"decode7","content":"pub fn decode7(\n  constructor: fn(a, b, c, d, e, f, g) -&gt; h,\n  t1: FieldDecoder(a),\n  t2: FieldDecoder(b),\n  t3: FieldDecoder(c),\n  t4: FieldDecoder(d),\n  t5: FieldDecoder(e),\n  t6: FieldDecoder(f),\n  t7: FieldDecoder(g),\n) -&gt; Decoder(h)\n\nSynonyms:\njson/blueprint.decode7\njson/blueprint decode7","url":"json/blueprint.html#decode7"},{"doc":"json/blueprint","title":"decode8","content":"pub fn decode8(\n  constructor: fn(a, b, c, d, e, f, g, h) -&gt; i,\n  t1: FieldDecoder(a),\n  t2: FieldDecoder(b),\n  t3: FieldDecoder(c),\n  t4: FieldDecoder(d),\n  t5: FieldDecoder(e),\n  t6: FieldDecoder(f),\n  t7: FieldDecoder(g),\n  t8: FieldDecoder(h),\n) -&gt; Decoder(i)\n\nSynonyms:\njson/blueprint.decode8\njson/blueprint decode8","url":"json/blueprint.html#decode8"},{"doc":"json/blueprint","title":"decode9","content":"pub fn decode9(\n  constructor: fn(a, b, c, d, e, f, g, h, i) -&gt; j,\n  t1: FieldDecoder(a),\n  t2: FieldDecoder(b),\n  t3: FieldDecoder(c),\n  t4: FieldDecoder(d),\n  t5: FieldDecoder(e),\n  t6: FieldDecoder(f),\n  t7: FieldDecoder(g),\n  t8: FieldDecoder(h),\n  t9: FieldDecoder(i),\n) -&gt; Decoder(j)\n\nSynonyms:\njson/blueprint.decode9\njson/blueprint decode9","url":"json/blueprint.html#decode9"},{"doc":"json/blueprint","title":"encode_optional_field","content":"pub fn encode_optional_field(\n  fields fields: List(#(String, Json)),\n  name name: String,\n  maybe value: Option(a),\n  encoder encode_fn: fn(a) -&gt; Json,\n) -&gt; List(#(String, Json))\n Adds an optional field to a list of key-value pairs that will be used to create a JSON object.\n This is particularly useful when defining JSON encoder to pair up to the `optional_field` decoder.\n\n ## Example\n \n fn tree_decoder() {\n   blueprint.union_type_decoder([\n     #(\n       &quot;node&quot;,\n       blueprint.decode3(\n         Node,\n         blueprint.field(&quot;value&quot;, blueprint.int()),\n         // This is an optional field for an optional value\n         blueprint.optional_field(&quot;left&quot;, blueprint.self_decoder(tree_decoder)),\n         // And this is a required field with an optional value\n         blueprint.field(\n           &quot;right&quot;,\n           blueprint.optional(blueprint.self_decoder(tree_decoder)),\n         ),\n       ),\n     ),\n   ])\n   |&gt; blueprint.reuse_decoder\n }\n\n fn encode_tree(tree: Tree) -&gt; json.Json {\n   blueprint.union_type_encoder(tree, fn(node) {\n     case node {\n       Node(value, left, right) -&gt; #(\n         &quot;node&quot;,\n         [\n           #(&quot;value&quot;, json.int(value)),\n           // This is a required field with an optional value\n           #(&quot;right&quot;, json.nullable(right, encode_tree)),\n         ]\n           // And this is an optional field for an optional value\n           |&gt; blueprint.encode_optional_field(&quot;left&quot;, left, encode_tree)\n           |&gt; json.object(),\n       )\n     }\n   })\n }\n \nSynonyms:\njson/blueprint.encode_optional_field\njson/blueprint encode_optional_field","url":"json/blueprint.html#encode_optional_field"},{"doc":"json/blueprint","title":"encode_tuple2","content":"pub fn encode_tuple2(\n  tuple tuple: #(a, b),\n  first encode1: fn(a) -&gt; Json,\n  second encode2: fn(b) -&gt; Json,\n) -&gt; Json\n\nSynonyms:\njson/blueprint.encode_tuple2\njson/blueprint encode_tuple2","url":"json/blueprint.html#encode_tuple2"},{"doc":"json/blueprint","title":"encode_tuple3","content":"pub fn encode_tuple3(\n  tuple tuple: #(a, b, c),\n  first encode1: fn(a) -&gt; Json,\n  second encode2: fn(b) -&gt; Json,\n  third encode3: fn(c) -&gt; Json,\n) -&gt; Json\n\nSynonyms:\njson/blueprint.encode_tuple3\njson/blueprint encode_tuple3","url":"json/blueprint.html#encode_tuple3"},{"doc":"json/blueprint","title":"encode_tuple4","content":"pub fn encode_tuple4(\n  tuple tuple: #(a, b, c, d),\n  first encode1: fn(a) -&gt; Json,\n  second encode2: fn(b) -&gt; Json,\n  third encode3: fn(c) -&gt; Json,\n  fourth encode4: fn(d) -&gt; Json,\n) -&gt; Json\n\nSynonyms:\njson/blueprint.encode_tuple4\njson/blueprint encode_tuple4","url":"json/blueprint.html#encode_tuple4"},{"doc":"json/blueprint","title":"encode_tuple5","content":"pub fn encode_tuple5(\n  tuple tuple: #(a, b, c, d, e),\n  first encode1: fn(a) -&gt; Json,\n  second encode2: fn(b) -&gt; Json,\n  third encode3: fn(c) -&gt; Json,\n  fourth encode4: fn(d) -&gt; Json,\n  fifth encode5: fn(e) -&gt; Json,\n) -&gt; Json\n\nSynonyms:\njson/blueprint.encode_tuple5\njson/blueprint encode_tuple5","url":"json/blueprint.html#encode_tuple5"},{"doc":"json/blueprint","title":"encode_tuple6","content":"pub fn encode_tuple6(\n  tuple tuple: #(a, b, c, d, e, f),\n  first encode1: fn(a) -&gt; Json,\n  second encode2: fn(b) -&gt; Json,\n  third encode3: fn(c) -&gt; Json,\n  fourth encode4: fn(d) -&gt; Json,\n  fifth encode5: fn(e) -&gt; Json,\n  sixth encode6: fn(f) -&gt; Json,\n) -&gt; Json\n\nSynonyms:\njson/blueprint.encode_tuple6\njson/blueprint encode_tuple6","url":"json/blueprint.html#encode_tuple6"},{"doc":"json/blueprint","title":"encode_tuple7","content":"pub fn encode_tuple7(\n  tuple tuple: #(a, b, c, d, e, f, g),\n  first encode1: fn(a) -&gt; Json,\n  second encode2: fn(b) -&gt; Json,\n  third encode3: fn(c) -&gt; Json,\n  fourth encode4: fn(d) -&gt; Json,\n  fifth encode5: fn(e) -&gt; Json,\n  sixth encode6: fn(f) -&gt; Json,\n  seventh encode7: fn(g) -&gt; Json,\n) -&gt; Json\n\nSynonyms:\njson/blueprint.encode_tuple7\njson/blueprint encode_tuple7","url":"json/blueprint.html#encode_tuple7"},{"doc":"json/blueprint","title":"encode_tuple8","content":"pub fn encode_tuple8(\n  tuple tuple: #(a, b, c, d, e, f, g, h),\n  first encode1: fn(a) -&gt; Json,\n  second encode2: fn(b) -&gt; Json,\n  third encode3: fn(c) -&gt; Json,\n  fourth encode4: fn(d) -&gt; Json,\n  fifth encode5: fn(e) -&gt; Json,\n  sixth encode6: fn(f) -&gt; Json,\n  seventh encode7: fn(g) -&gt; Json,\n  eighth encode8: fn(h) -&gt; Json,\n) -&gt; Json\n\nSynonyms:\njson/blueprint.encode_tuple8\njson/blueprint encode_tuple8","url":"json/blueprint.html#encode_tuple8"},{"doc":"json/blueprint","title":"encode_tuple9","content":"pub fn encode_tuple9(\n  tuple tuple: #(a, b, c, d, e, f, g, h, i),\n  first encode1: fn(a) -&gt; Json,\n  second encode2: fn(b) -&gt; Json,\n  third encode3: fn(c) -&gt; Json,\n  fourth encode4: fn(d) -&gt; Json,\n  fifth encode5: fn(e) -&gt; Json,\n  sixth encode6: fn(f) -&gt; Json,\n  seventh encode7: fn(g) -&gt; Json,\n  eighth encode8: fn(h) -&gt; Json,\n  ninth encode9: fn(i) -&gt; Json,\n) -&gt; Json\n\nSynonyms:\njson/blueprint.encode_tuple9\njson/blueprint encode_tuple9","url":"json/blueprint.html#encode_tuple9"},{"doc":"json/blueprint","title":"enum_type_decoder","content":"pub fn enum_type_decoder(\n  constructor_decoders decoders: List(#(String, a)),\n) -&gt; Decoder(a)\n Function to define a decoder for enum types (unions where constructors have no arguments).\n The function takes a list of tuples containing the string representation and the corresponding enum value.\n\n&gt; [!IMPORTANT]  \n&gt; Make sure to add tests for every possible enum value because it is not possible to check for exhaustiveness.\n\n ## Example\n \n type Color {\n   Red\n   Green\n   Blue\n }\n\n let color_decoder = enum_type_decoder([\n   #(&quot;red&quot;, Red),\n   #(&quot;green&quot;, Green),\n   #(&quot;blue&quot;, Blue),\n ])\n \n\nSynonyms:\njson/blueprint.enum_type_decoder\njson/blueprint enum_type_decoder","url":"json/blueprint.html#enum_type_decoder"},{"doc":"json/blueprint","title":"enum_type_encoder","content":"pub fn enum_type_encoder(\n  value of: a,\n  encoder_fn encoder_fn: fn(a) -&gt; String,\n) -&gt; Json\n Function to encode an enum type (unions where constructors have no arguments) into a JSON object.\n The function takes a value and an encoder function that returns the string representation of the enum value.\n\n&gt; [!IMPORTANT]  \n&gt; Make sure to update the decoder function accordingly.\n\n ## Example\n \n type Color {\n   Red\n   Green\n   Blue\n }\n\n let color_encoder = enum_type_encoder(fn(color) {\n   case color {\n     Red -&gt; &quot;red&quot;\n     Green -&gt; &quot;green&quot;\n     Blue -&gt; &quot;blue&quot;\n   }\n })\n \n\nSynonyms:\njson/blueprint.enum_type_encoder\njson/blueprint enum_type_encoder","url":"json/blueprint.html#enum_type_encoder"},{"doc":"json/blueprint","title":"field","content":"pub fn field(\n  named name: String,\n  of inner_type: Decoder(a),\n) -&gt; FieldDecoder(a)\n\nSynonyms:\njson/blueprint.field\njson/blueprint field","url":"json/blueprint.html#field"},{"doc":"json/blueprint","title":"float","content":"pub fn float() -&gt; Decoder(Float)\n\nSynonyms:\njson/blueprint.float\njson/blueprint float","url":"json/blueprint.html#float"},{"doc":"json/blueprint","title":"generate_json_schema","content":"pub fn generate_json_schema(decoder: Decoder(a)) -&gt; Json\n\nSynonyms:\njson/blueprint.generate_json_schema\njson/blueprint generate_json_schema","url":"json/blueprint.html#generate_json_schema"},{"doc":"json/blueprint","title":"get_dynamic_decoder","content":"pub fn get_dynamic_decoder(\n  decoder: Decoder(a),\n) -&gt; fn(Dynamic) -&gt; Result(a, List(DecodeError))\n\nSynonyms:\njson/blueprint.get_dynamic_decoder\njson/blueprint get_dynamic_decoder","url":"json/blueprint.html#get_dynamic_decoder"},{"doc":"json/blueprint","title":"int","content":"pub fn int() -&gt; Decoder(Int)\n\nSynonyms:\njson/blueprint.int\njson/blueprint int","url":"json/blueprint.html#int"},{"doc":"json/blueprint","title":"list","content":"pub fn list(of decoder_type: Decoder(a)) -&gt; Decoder(List(a))\n\nSynonyms:\njson/blueprint.list\njson/blueprint list","url":"json/blueprint.html#list"},{"doc":"json/blueprint","title":"map","content":"pub fn map(\n  decoder decoder: Decoder(a),\n  over foo: fn(a) -&gt; b,\n) -&gt; Decoder(b)\n\nSynonyms:\njson/blueprint.map\njson/blueprint map","url":"json/blueprint.html#map"},{"doc":"json/blueprint","title":"optional","content":"pub fn optional(of decode: Decoder(a)) -&gt; Decoder(Option(a))\n Creates a decoder that can handle `null` values by wrapping the result in an `Option` type.\n When the value is `null`, it returns `None`. Otherwise, it uses the provided decoder\n to decode the value and wraps the result in `Some`. If you need the decoder to handle a possible missing field\n (i.e., the field is absent from the JSON), use the `optional_field` function instead.\n\n ## Example\n \n type User {\n   User(name: String, age: Option(Int))\n }\n\n let decoder = decode2(\n   User,\n   field(&quot;name&quot;, string()),\n   field(&quot;age&quot;, optional(int()))  // Will handle &quot;age&quot;: null\n )\n\n // These JSON strings will decode successfully:\n // {&quot;name&quot;: &quot;Alice&quot;, &quot;age&quot;: 25}  -&gt; User(&quot;Alice&quot;, Some(25))\n // {&quot;name&quot;: &quot;Bob&quot;, &quot;age&quot;: null}  -&gt; User(&quot;Bob&quot;, None)\n \n\nSynonyms:\njson/blueprint.optional\njson/blueprint optional","url":"json/blueprint.html#optional"},{"doc":"json/blueprint","title":"optional_field","content":"pub fn optional_field(\n  named name: String,\n  of inner_type: Decoder(a),\n) -&gt; FieldDecoder(Option(a))\n Decode a field that can be missing or have a `null` value into an `Option` type.\n This function is useful when you want to handle both cases where a field is absent from the JSON\n or when it&#39;s explicitly set to `null`.\n\n If you only need to handle fields that are present but might be `null`, use the `optional` function instead.\n\n ## Example\n \n type User {\n   User(name: String, age: Option(Int))\n }\n\n let decoder = decode2(\n   User,\n   field(&quot;name&quot;, string()),\n   optional_field(&quot;age&quot;, int())  // Will handle both missing &quot;age&quot; field and &quot;age&quot;: null\n )\n\n // All these JSON strings will decode successfully:\n // {&quot;name&quot;: &quot;Alice&quot;, &quot;age&quot;: 25}     -&gt; User(&quot;Alice&quot;, Some(25))\n // {&quot;name&quot;: &quot;Bob&quot;, &quot;age&quot;: null}     -&gt; User(&quot;Bob&quot;, None)\n // {&quot;name&quot;: &quot;Charlie&quot;}              -&gt; User(&quot;Charlie&quot;, None)\n \n\nSynonyms:\njson/blueprint.optional_field\njson/blueprint optional_field","url":"json/blueprint.html#optional_field"},{"doc":"json/blueprint","title":"reuse_decoder","content":"pub fn reuse_decoder(decoder: Decoder(a)) -&gt; Decoder(a)\n Creates a reusable version of a decoder that can be used multiple times in a schema\n without duplicating the schema definition.\n\n The function:\n 1. Creates a unique reference name based on the schema&#39;s hash\n 2. Moves the original schema into the `$defs` section\n 3. Returns a new decoder that references the schema via `$ref`\n\n ## Example\n \n type Person {\n   Person(name: String, friends: List(Pet))\n }\n\n type Pet {\n   Pet(name: String)\n }\n\n\n let pet_decoder = reuse_decoder(\n   decode2(\n     Pet,\n     field(&quot;name&quot;, string()),\n   )\n )\n\n let person_decoder = reuse_decoder(\n   decode2(\n     Person,\n     field(&quot;name&quot;, string()),\n     field(&quot;friends&quot;, list(pet_decoder))\n   )\n )\n \n\nSynonyms:\njson/blueprint.reuse_decoder\njson/blueprint reuse_decoder","url":"json/blueprint.html#reuse_decoder"},{"doc":"json/blueprint","title":"self_decoder","content":"pub fn self_decoder(lazy: fn() -&gt; Decoder(a)) -&gt; Decoder(a)\n Creates a decoder for recursive data types by allowing self-referential definitions.\n This is useful when you have types that contain themselves, like trees or linked lists.\n\n The function takes a lazy decoder (a function that returns a decoder) to break the\n recursive dependency cycle. The returned decoder uses a JSON Schema reference &quot;#&quot;\n to point to the root schema definition.\n\n &gt; ❗ _**IMPORTANT**_ \n &gt; Add the reuse_decoder when there are nested recursive types so\n &gt; the schema references (`#`) get rewritten correctly and self-references from the\n &gt; different types don&#39;t get mixed up. As a recommendation, always add it when\n &gt; decoding recursive types.\n\n ## Example\n \n // A binary tree type that can contain itself\n pub type Tree {\n   Node(value: Int, left: Option(Tree), right: Option(Tree))\n   Leaf(value: Int)\n }\n\n // Create a recursive decoder for the Tree type\n pub fn tree_decoder() -&gt; Decoder(Tree) {\n   // Use union_type_decoder for handling different variants\n   union_type_decoder([\n     #(&quot;leaf&quot;, decode1(Leaf, field(&quot;value&quot;, int()))),\n     #(&quot;node&quot;, decode3(\n       Node,\n       field(&quot;value&quot;, int()),\n       // Use self_decoder to handle recursive fields\n       field(&quot;left&quot;, optional(self_decoder(tree_decoder))),\n       field(&quot;right&quot;, optional(self_decoder(tree_decoder))),\n     )),\n   ])\n   |&gt; reuse_decoder\n }\n \n\nSynonyms:\njson/blueprint.self_decoder\njson/blueprint self_decoder","url":"json/blueprint.html#self_decoder"},{"doc":"json/blueprint","title":"string","content":"pub fn string() -&gt; Decoder(String)\n\nSynonyms:\njson/blueprint.string\njson/blueprint string","url":"json/blueprint.html#string"},{"doc":"json/blueprint","title":"tuple2","content":"pub fn tuple2(\n  first decode1: Decoder(a),\n  second decode2: Decoder(b),\n) -&gt; Decoder(#(a, b))\n\nSynonyms:\njson/blueprint.tuple2\njson/blueprint tuple2","url":"json/blueprint.html#tuple2"},{"doc":"json/blueprint","title":"tuple3","content":"pub fn tuple3(\n  first decode1: Decoder(a),\n  second decode2: Decoder(b),\n  third decode3: Decoder(c),\n) -&gt; Decoder(#(a, b, c))\n\nSynonyms:\njson/blueprint.tuple3\njson/blueprint tuple3","url":"json/blueprint.html#tuple3"},{"doc":"json/blueprint","title":"tuple4","content":"pub fn tuple4(\n  first decode1: Decoder(a),\n  second decode2: Decoder(b),\n  third decode3: Decoder(c),\n  fourth decode4: Decoder(d),\n) -&gt; Decoder(#(a, b, c, d))\n\nSynonyms:\njson/blueprint.tuple4\njson/blueprint tuple4","url":"json/blueprint.html#tuple4"},{"doc":"json/blueprint","title":"tuple5","content":"pub fn tuple5(\n  first decode1: Decoder(a),\n  second decode2: Decoder(b),\n  third decode3: Decoder(c),\n  fourth decode4: Decoder(d),\n  fifth decode5: Decoder(e),\n) -&gt; Decoder(#(a, b, c, d, e))\n\nSynonyms:\njson/blueprint.tuple5\njson/blueprint tuple5","url":"json/blueprint.html#tuple5"},{"doc":"json/blueprint","title":"tuple6","content":"pub fn tuple6(\n  first decode1: Decoder(a),\n  second decode2: Decoder(b),\n  third decode3: Decoder(c),\n  fourth decode4: Decoder(d),\n  fifth decode5: Decoder(e),\n  sixth decode6: Decoder(f),\n) -&gt; Decoder(#(a, b, c, d, e, f))\n\nSynonyms:\njson/blueprint.tuple6\njson/blueprint tuple6","url":"json/blueprint.html#tuple6"},{"doc":"json/blueprint","title":"union_type_decoder","content":"pub fn union_type_decoder(\n  constructor_decoders decoders: List(#(String, Decoder(a))),\n) -&gt; Decoder(a)\n Function to defined a decoder for a union types.\n The function takes a list of decoders for each possible type of the union.\n\n&gt; [!IMPORTANT]  \n&gt; Make sure to add tests for every possible type of the union because it is not possible to check for exhaustiveness in the case.\n\n ## Example\n \n type Shape {\n   Circle(Float)\n   Rectangle(Float, Float)\n }\n\n let shape_decoder = union_type_decoder([\n   #(&quot;circle&quot;, decode1(Circle, field(&quot;radius&quot;, float()))),\n   #(&quot;rectangle&quot;, decode2(Rectangle, \n     field(&quot;width&quot;, float()),\n     field(&quot;height&quot;, float())\n   ))\n ])\n \n\nSynonyms:\njson/blueprint.union_type_decoder\njson/blueprint union_type_decoder","url":"json/blueprint.html#union_type_decoder"},{"doc":"json/blueprint","title":"union_type_encoder","content":"pub fn union_type_encoder(\n  value of: a,\n  encoder_fn encoder_fn: fn(a) -&gt; #(String, Json),\n) -&gt; Json\n Function to encode a union type into a JSON object.\n The function takes a value and an encoder function that returns a tuple of the type name and the JSON value.\n\n&gt; [!IMPORTANT]  \n&gt; Make sure to update the decoder function accordingly.\n\n ## Example\n \n type Shape {\n   Circle(Float)\n   Rectangle(Float, Float)\n }\n\n let shape_encoder = union_type_encoder(fn(shape) {\n   case shape {\n     Circle(radius) -&gt; #(&quot;circle&quot;, json.object([#(&quot;radius&quot;, json.float(radius))]))\n     Rectangle(width, height) -&gt; #(\n       &quot;rectangle&quot;,\n       json.object([\n         #(&quot;width&quot;, json.float(width)),\n         #(&quot;height&quot;, json.float(height))\n       ])\n     )\n   }\n })\n \n\n\nSynonyms:\njson/blueprint.union_type_encoder\njson/blueprint union_type_encoder","url":"json/blueprint.html#union_type_encoder"},{"doc":"json/blueprint","title":"json/blueprint","content":"","url":"json/blueprint.html"}]);